import { SharedContext } from '../types';
import type * as vscode from '@volar/language-service';
export declare function register(ctx: SharedContext): (uri: string, positions: vscode.Position[]) => vscode.SelectionRange[];
