import { Service } from '@volar/language-service';
import type * as ts from 'typescript/lib/tsserverlibrary';
export default function (): Service;
export declare function createAddComponentToOptionEdit(ts: typeof import('typescript/lib/tsserverlibrary'), ast: ts.SourceFile, componentName: string): {
    range: import("@vue/language-core").TextRange;
    newText: string;
} | undefined;
